//
// ColTensor Class header file
//

#ifndef ColTensor_hh
#define ColTensor_hh

#include <cassert>

namespace FemLib {

class ColTensor {
    public:

    // constructors and destructors
 
        ColTensor()
            { for (int i=0 ; i<6 ; ++i) data[i] = 0.0 ; } ;

        ColTensor(double xx,double yy,double zz,
                  double xy,double yz,double zx)
            { data[0] = xx ;  data[1] = yy ;  data[2] = zz ;
              data[3] = xy ;  data[4] = yz ;  data[5] = zx ; } ;

    // access

        double xx() const { return(data[0]) ; } ;
        double yy() const { return(data[1]) ; } ;
        double zz() const { return(data[2]) ; } ;
        double xy() const { return(data[3]) ; } ;
        double yz() const { return(data[4]) ; } ;
        double zx() const { return(data[5]) ; } ;

    // operators

        double &operator[] (const int i)
            { assert(i < 6) ; return(data[i]) ; } ;

        double operator[] (const int i) const
            { assert(i < 6) ; return(data[i]) ; } ;

        ColTensor operator *= (const double &op2)
            { for (int i=0 ; i<6 ; ++i) data[i] *= op2 ;
              return(*this) ; } ;

        ColTensor operator = (int val)
            { for (int i=0 ; i<6 ; ++i) data[i] = val ;
              return(*this) ; } ;

        // ColTensor PrincipalValues() const ;

    private:

        double data[6] ;

    friend double operator * (const ColTensor &op1,
                              const ColTensor &op2) ;
    friend ColTensor operator - (const ColTensor &op1,
                                 const ColTensor &op2) ;
    friend ColTensor operator + (const ColTensor &op1,
                                 const ColTensor &op2) ;
} ;

inline ColTensor operator * (const ColTensor &op1,const double &op2)
{
    ColTensor op = op1 ;
    op *= op2 ;
    return(op) ;
}

inline ColTensor operator * (const double &op1,const ColTensor &op2)
{
    ColTensor op = op2 ;
    op *= op1 ;
    return(op) ;
}

inline double operator * (const ColTensor &op1,const ColTensor &op2)
{
    return(op1.data[0]*op2.data[0] + op1.data[1]*op2.data[1] +
           op1.data[2]*op2.data[2] + op1.data[3]*op2.data[3] +
           op1.data[4]*op2.data[4] + op1.data[5]*op2.data[5]) ;
}

inline ColTensor operator - (const ColTensor &op1,const ColTensor &op2)
{
    return(ColTensor(op1.data[0]-op2.data[0],op1.data[1]-op2.data[1],
                     op1.data[2]-op2.data[2],op1.data[3]-op2.data[3],
                     op1.data[4]-op2.data[4],op1.data[5]-op2.data[5])) ;
}
                      
inline ColTensor operator + (const ColTensor &op1,const ColTensor &op2)
{
    return(ColTensor(op2.data[0]+op2.data[0],op2.data[1]+op2.data[1],
                     op2.data[2]+op2.data[2],op2.data[3]+op2.data[3],
                     op2.data[4]+op2.data[4],op2.data[5]+op2.data[5])) ;
}
                      
typedef ColTensor StressVec ;
typedef ColTensor StrainVec ;

} // namespace

#endif
