
import math

class ColTensor:

    """Class to represent a 2nd order tensor in column format"""

    def __init__(self,xx,yy,zz,xy,yz,zx):
        """Constructor for a 2nd order column tensor class"""
        self.data = [xx,yy,zz,xy,yz,zx]

    def xx(self):
        return self.data[0]

    def yy(self):
        return self.data[1]

    def zz(self):
        return self.data[2]

    def xy(self):
        return self.data[3]

    def yz(self):
        return self.data[4]

    def zx(self):
        return self.data[5]

    def __getitem__(self,key):
        if key < 0 or key > 5: raise IndexError
        return self.data[key]

    def __setitem__(self,key,value):
        if key < 0 or key > 5: raise IndexError
        self.data[key] = value

    def __iadd__(self,other):
        if isinstance(other,ColTensor):
            for i in xrange(6):
                data[i] += other.data[i]
        else:
            for i in xrange(6):
                data[i] += other
        return(self)

    def __isub__(self,other):
        if isinstance(other,ColTensor):
            for i in xrange(6):
                data[i] -= other.data[i]
        else:
            for i in xrange(6):
                data[i] -= other
        return(self)

    def __imul__(self,other):
        for i in xrange(6):
            data[i] *= other
        return(self)

    def __idiv__(self,other):
        for i in xrange(6):
            data[i] /= other
        return(self)

    def __neg__(self):
        return ColTensor(-self.data[0],-self.data[1],-self.data[2],
                         -self.data[3],-self.data[4],-self.data[5])

    def __add__(self,other):
        if isinstance(other,ColTensor):
            return ColTensor(self.data[0]+other.data[0],
                             self.data[1]+other.data[1],
                             self.data[2]+other.data[2],
                             self.data[3]+other.data[3],
                             self.data[4]+other.data[4],
                             self.data[5]+other.data[5])
        else:
            return ColTensor(self.data[0]+other,
                             self.data[1]+other,
                             self.data[2]+other,
                             self.data[3]+other,
                             self.data[4]+other,
                             self.data[5]+other)
 

    def __sub__(self,other):
        if isinstance(other,ColTensor):
            return ColTensor(self.data[0]-other.data[0],
                             self.data[1]-other.data[1],
                             self.data[2]-other.data[2],
                             self.data[3]-other.data[3],
                             self.data[4]-other.data[4],
                             self.data[5]-other.data[5])
        else:
            return ColTensor(self.data[0]-other,
                             self.data[1]-other,
                             self.data[2]-other,
                             self.data[3]-other,
                             self.data[4]-other,
                             self.data[5]-other)

    def __rsub__(self,other):
        return ColTensor(other-self.data[0],
                         other-self.data[1],
                         other-self.data[2],
                         other-self.data[3],
                         other-self.data[4],
                         other-self.data[5])

    def __mul__(self,other):
        return ColTensor(self.data[0]*other,
                         self.data[1]*other,
                         self.data[2]*other,
                         self.data[3]*other,
                         self.data[4]*other,
                         self.data[5]*other)

    def __rmul__(self,other):
        return ColTensor(self.data[0]*other,
                         self.data[1]*other,
                         self.data[2]*other,
                         self.data[3]*other,
                         self.data[4]*other,
                         self.data[5]*other)

    def __div__(self,other):
        return ColTensor(self.data[0]/other,
                         self.data[1]/other,
                         self.data[2]/other,
                         self.data[3]/other,
                         self.data[4]/other,
                         self.data[5]/other)

    def __repr__(self):
        return "(%g %g %g %g %g %g)" % (self.data[0],self.data[1],
              self.data[2],self.data[3],self.data[4],self.data[5])
