#!/usr/local/bin/python

import sys
import string
import FemLib
import DMatrix

from Vec3D import Vec3D
from Numeric import *
from LinearAlgebra import *
from pprint import pprint

sys.path.append('/afs/.msc.cornell.edu/i386_linux22/usr/local/pub/dm/lib/python2.1/site-packages')
import Sybase


class DBPartialFemModel:
    """The DBFemModel class acts as a container to hold a description
       of a mesh and associated elements and boundary conditions."""

    # constructor that reads a model description from database

    def __init__(self,server,database,edgeToDecorate,lengthScale):
        """Reads the mesh surrounding an edge from the database

        """

        # connect to database
        username = "cmi"
        password = "cmi4now"
        db = Sybase.connect(server,username,password,database)

        # find the crack nodes
        queryString = "SELECT v0, v1 FROM Edges WHERE EdgeID = " + str(edgeToDecorate)
        crackNodeData = db.execute(queryString)
        self.crackNodes = []
        self.crackNodes.append(crackNodeData[0][0][0])
        self.crackNodes.append(crackNodeData[0][0][1])
            
        # read the node coordinates
        self.nodes = {}
        queryString = """
        SELECT DISTINCT A.VertexID, B.x, B.y, B.z
        FROM TetrahedronVertices AS A
        JOIN Vertices AS B ON A.VertexID = B.VertexID
        WHERE A.TetID IN (SELECT TetID
        FROM TetrahedronVertices
        WHERE VertexID IN (""" + str(self.crackNodes[0]) + ", " + str(self.crackNodes[1]) + """)
        GROUP BY TetID
        HAVING COUNT(VertexID) = 2)
        """
        nodeData = db.execute(queryString)
        for i in xrange(len(nodeData[0])): #rows in table
            self.nodes[nodeData[0][i][0]] = (lengthScale*nodeData[0][i][1],lengthScale*nodeData[0][i][2],lengthScale*nodeData[0][i][3])

        # the element data

        self.elem = {}
        queryString = """
        SELECT TetID, v0, v1, v2, v3
        FROM
        Tetrahedra WHERE
        TetID IN (SELECT TetID
        FROM TetrahedronVertices
        WHERE VertexID IN (""" + str(self.crackNodes[0]) + ", " + str(self.crackNodes[1]) + """) 
        GROUP BY TetID
        HAVING COUNT(VertexID) = 2)
        """
        elementData = db.execute(queryString)
        self.edgeNodeMap = {}
        for i in xrange(len(elementData[0])):
            id   = elementData[0][i][0]
            type = "Tet2I"  #XXX FOR NOW
            mat  = 0
            conn = []
            for node in elementData[0][i][1:]: conn.append(int(node))
            self.elem[id] = (type,mat,conn)
            cornerNodes = conn[:]
            # for some reason these need to be in a very specific order
            for i in xrange(1,len(cornerNodes)):
                for j in xrange(1,i+1):
                    newEdgeNodeID = self.GetEdgeNode((cornerNodes[i-j],cornerNodes[i]))
                    self.elem[id][2].append(newEdgeNodeID)

    def OrderedPair(self,pair):
        if pair[0] > pair[1]:
            orderedPair = (pair[1],pair[0])
        else:
            orderedPair = pair
        return orderedPair

    def GetEdgeNode(self, pair):
        # the dictionary which maps pairs of corner nodes to
        # edge nodes assumes that the pairs are ordered
        orderedPair = self.OrderedPair(pair)

        # if this pair is already in the dictionary then we just look up the
        # value
        if self.edgeNodeMap.has_key(orderedPair):
            return self.edgeNodeMap[orderedPair]
        else:
            endVert0 = self.nodes[orderedPair[0]]
            endVert1 = self.nodes[orderedPair[1]]
            newEdgeNodeID = len(self.nodes.keys()) #self.GetNewNodeID()
            newEdgeNodePos = (0.5 * (endVert0[0] + endVert1[0]), 0.5 * (endVert0[1] + endVert1[1]),0.5 * (endVert0[2] + endVert1[2]))
            
            self.nodes[newEdgeNodeID] = newEdgeNodePos
            self.edgeNodeMap[orderedPair] = newEdgeNodeID
            return newEdgeNodeID
 
    def __repr__(self):
        list = ["[ FEM Model:",""]
        list.append("  Number of Nodes:     %d" % self.num_nodes)
        list.append("  Number of Materials: %d" % self.num_mats)
        list.append("  Number of Elements:  %d" % self.num_elems)
        list.append("")

        list.append("  Nodes:")
        items = self.nodes.items()
        items.sort()
        for item in items:
            list.append("  %4d  %11.4g  %11.4g  %11.4g" %
                        (item[0],item[1][0],item[1][1],item[1][2]))
        list.append("")

        list.append("  Elements:")
        items = self.elem.items()
        for item in items:
            tmp = ["  %4d  %s %4d  " % (item[0],item[1][0],item[1][1])]
            for nd in item[1][2]:
                tmp.append(" %4d" % nd)
            list.append(string.joinfields(tmp))
        list.append("")

        list.append("]")
        return string.joinfields(list,"\n")

    def GetElemCoords(self,i):
        """Return a list of nodal coordinates for element i"""
        coords = []
        for node in self.elem[i][2]:
            vals = self.nodes[node]
            coords.append(Vec3D(vals[0],vals[1],vals[2]))
        return coords

    def GetNodeCoord(self,id):
        return self.nodes[id]

    def NumNodes(self):
        return len(self.nodes.keys())

    def NumElems(self):
        return len(self.elem.keys())

    def GetElemData(self,i):
        return self.elem[i]

    def GetNodeIds(self):
        return self.nodes.keys()

    def GetElemIds(self):
        return self.elem.keys()

    def GetCrackNodes(self):
        return self.crackNodes

    def GetNewNodeID(self):
        ID = len(self.nodes.keys()) + 1
        while 1:
            if ID in self.nodes.keys():
                ID = ID + 1
            else:
                return ID
