//
// Integration points implementation file
//
// Revision -
//   $Revision: 1.2 $  $Date: 2001/12/19 02:47:16 $  $Author: wash $
//
 
#include "IntegrationPts.hh"

namespace FemLib {

IntegrationPts::IntegrationPts()
{
    // triangle, 3 points

    Tri3Pt = TVector<Vec3D>(3) ;
    Tri3Pt[0] = Vec3D(0.5,0.5,0.0) ;
    Tri3Pt[1] = Vec3D(0.0,0.5,0.0) ;
    Tri3Pt[2] = Vec3D(0.5,0.0,0.0) ;

    Tri3Wt = TVector<double>(3) ;
    Tri3Wt[0] = 0.166666667 ;
    Tri3Wt[1] = 0.166666667 ;
    Tri3Wt[2] = 0.166666667 ;

    // triangle, 4 points

    Tri4Pt = TVector<Vec3D>(4) ;
    Tri4Pt[0] = Vec3D(0.333333333,0.333333333,0.0) ;
    Tri4Pt[1] = Vec3D(0.6,0.2,0.0) ;
    Tri4Pt[2] = Vec3D(0.2,0.6,0.0) ;
    Tri4Pt[3] = Vec3D(0.2,0.2,0.0) ;

    Tri4Wt = TVector<double>(4) ;
    Tri4Wt[0] = -0.28125 ;
    Tri4Wt[1] =  0.260416667 ;
    Tri4Wt[2] =  0.260416667 ;
    Tri4Wt[3] =  0.260416667 ;

    // triangle, 7 points

    const double val_7pt1 = 0.0597158717 ;
    const double val_7pt2 = 0.4701420641 ;
    const double val_7pt3 = 0.7974269853 ;
    const double val_7pt4 = 0.1012865073 ;
    Tri7Pt = TVector<Vec3D>(7) ;
    Tri7Pt[0] = Vec3D(0.333333333,0.333333333,0.0) ;
    Tri7Pt[1] = Vec3D(val_7pt1,val_7pt2,0.0) ;
    Tri7Pt[2] = Vec3D(val_7pt2,val_7pt1,0.0) ;
    Tri7Pt[3] = Vec3D(val_7pt2,val_7pt2,0.0) ;
    Tri7Pt[4] = Vec3D(val_7pt3,val_7pt4,0.0) ;
    Tri7Pt[5] = Vec3D(val_7pt4,val_7pt3,0.0) ;
    Tri7Pt[6] = Vec3D(val_7pt4,val_7pt4,0.0) ;

    Tri7Wt = TVector<double>(7) ;
    Tri7Wt[0] = 0.1125 ;
    Tri7Wt[1] = 0.06619707635 ;
    Tri7Wt[2] = 0.06619707635 ;
    Tri7Wt[3] = 0.06619707635 ;
    Tri7Wt[4] = 0.06296959025 ;
    Tri7Wt[5] = 0.06296959025 ;
    Tri7Wt[6] = 0.06296959025 ;

    // quadrilateral, 1 point

    Quad1Pt = TVector<Vec3D>(1) ;
    Quad1Pt[0] = Vec3D(0.0,0.0,0.0) ;

    Quad1Wt = TVector<double>(1) ;
    Quad1Wt[0] = 4.0 ;

    // quadrilateral, 4 points

    const double val_4pt = 0.577350269189626 ;
    Quad4Pt = TVector<Vec3D>(4) ;
    Quad4Pt[0] = Vec3D(-val_4pt,-val_4pt,0.0) ;
    Quad4Pt[1] = Vec3D( val_4pt,-val_4pt,0.0) ;
    Quad4Pt[2] = Vec3D(-val_4pt, val_4pt,0.0) ;
    Quad4Pt[3] = Vec3D( val_4pt, val_4pt,0.0) ;

    Quad4Wt = TVector<double>(4) ;
    for (int i=0 ; i<4 ; ++i) Quad4Wt[i] = 1.0 ;

    // quadrilateral, 9 points

    const double val_9pt = 0.774596669241483 ;
    Quad9Pt = TVector<Vec3D>(9) ;
    Quad9Pt[0] = Vec3D(-val_9pt,-val_9pt,0.0) ;
    Quad9Pt[1] = Vec3D(     0.0,-val_9pt,0.0) ;
    Quad9Pt[2] = Vec3D( val_9pt,-val_9pt,0.0) ;
    Quad9Pt[3] = Vec3D(-val_9pt,     0.0,0.0) ;
    Quad9Pt[4] = Vec3D(     0.0,     0.0,0.0) ;
    Quad9Pt[5] = Vec3D( val_9pt,     0.0,0.0) ;
    Quad9Pt[6] = Vec3D(-val_9pt, val_9pt,0.0) ;
    Quad9Pt[7] = Vec3D(     0.0, val_9pt,0.0) ;
    Quad9Pt[8] = Vec3D( val_9pt, val_9pt,0.0) ;

    Quad9Wt = TVector<double>(9) ;
    Quad9Wt[0] = 0.308641975 ;
    Quad9Wt[1] = 0.493827160 ;
    Quad9Wt[2] = 0.308641975 ;
    Quad9Wt[3] = 0.493827160 ;
    Quad9Wt[4] = 0.790123457 ;
    Quad9Wt[5] = 0.493827160 ;
    Quad9Wt[6] = 0.308641975 ;
    Quad9Wt[7] = 0.493827160 ;
    Quad9Wt[8] = 0.308641975 ;
}

const TVector<Vec3D> &IntegrationPts::GetSurfaceCoords(
            SurfaceShape shape,int number) const
{
    if (shape == TRIANGLE) {
        assert((number == 3) || (number == 4) || (number == 7)) ;
        switch (number) {
            case 3: return(Tri3Pt) ;
            case 4: return(Tri4Pt) ;
            case 7: return(Tri7Pt) ;
        }
    } else if (shape == QUADRILATERAL) {
        assert((number == 1) || (number == 4) || (number == 9)) ;
        switch (number) {
            case 1: return(Quad1Pt) ;
            case 4: return(Quad4Pt) ;
            case 9: return(Quad9Pt) ;
        }
    }
    return(NullPt) ;
}

const TVector<double> &IntegrationPts::GetSurfaceWeights(
            SurfaceShape shape,int number) const
{
    if (shape == TRIANGLE) {
        assert((number == 3) || (number == 4) || (number == 7)) ;
        switch (number) {
            case 3: return(Tri3Wt) ;
            case 4: return(Tri4Wt) ;
            case 7: return(Tri7Wt) ;
        }
    } else if (shape == QUADRILATERAL) {
        assert((number == 1) || (number == 4) || (number == 9)) ;
        switch (number) {
            case 1: return(Quad1Wt) ;
            case 4: return(Quad4Wt) ;
            case 9: return(Quad9Wt) ;
        }
    }
    return(NullWt) ;
}

// ------------------------------------------------------
//  Implementation for the TetIntegration class

TetIntegrationPts *TetIntegrationPts::_instance = 0 ;

TetIntegrationPts *TetIntegrationPts::Instance()
{
    if (_instance == 0) {
        _instance = new TetIntegrationPts ;
    }
    return _instance ;
}

TetIntegrationPts::TetIntegrationPts() : IntegrationPts()
{
    // 1 point

    Tet1Pt = TVector<Vec3D>(1) ;
    Tet1Pt[0] = Vec3D(0.25,0.25,0.25) ;

    Tet1Wt = TVector<double>(1) ;
    Tet1Wt[0] = 0.166666667 ;

    // 4 points (degree 2)

    const double val_4pt_1 = 0.58541020 ;
    const double val_4pt_2 = 0.13819660 ;
 
    Tet4Pt = TVector<Vec3D>(4) ;
    Tet4Pt[0] = Vec3D(val_4pt_1,val_4pt_2,val_4pt_2) ;
    Tet4Pt[1] = Vec3D(val_4pt_2,val_4pt_1,val_4pt_2) ;
    Tet4Pt[2] = Vec3D(val_4pt_2,val_4pt_2,val_4pt_1) ;
    Tet4Pt[3] = Vec3D(val_4pt_2,val_4pt_2,val_4pt_2) ;

    Tet4Wt = TVector<double>(4) ;
    Tet4Wt[0] = 0.041666667 ;
    Tet4Wt[1] = 0.041666667 ;
    Tet4Wt[2] = 0.041666667 ;
    Tet4Wt[3] = 0.041666667 ;

    // 5 points (degree 3)

    const double val_5pt = 0.166666667 ;
 
    Tet5Pt = TVector<Vec3D>(5) ;
    Tet5Pt[0] = Vec3D(0.25,0.25,0.25) ;
    Tet5Pt[1] = Vec3D(    0.5,val_5pt,val_5pt) ;
    Tet5Pt[2] = Vec3D(val_5pt,    0.5,val_5pt) ;
    Tet5Pt[3] = Vec3D(val_5pt,val_5pt,    0.5) ;
    Tet5Pt[4] = Vec3D(val_5pt,val_5pt,val_5pt) ;

    Tet5Wt = TVector<double>(5) ;
    Tet5Wt[0] = -0.133333333 ;
    Tet5Wt[1] =  0.075 ;
    Tet5Wt[2] =  0.075 ;
    Tet5Wt[3] =  0.075 ;
    Tet5Wt[4] =  0.075 ;

    // 11 points (degree 4)

    const double val_11pt_1 = 0.071428571428571 ;
    const double val_11pt_2 = 0.785714285714257 ;
    const double val_11pt_3 = 0.399403576166799 ;
    const double val_11pt_4 = 0.100596423833200 ;
 
    Tet11Pt = TVector<Vec3D>(11) ;
    Tet11Pt[0]  = Vec3D(0.25,0.25,0.25) ;
    Tet11Pt[1]  = Vec3D(val_11pt_1,val_11pt_1,val_11pt_1) ;
    Tet11Pt[2]  = Vec3D(val_11pt_2,val_11pt_1,val_11pt_1) ;
    Tet11Pt[3]  = Vec3D(val_11pt_1,val_11pt_2,val_11pt_1) ;
    Tet11Pt[4]  = Vec3D(val_11pt_1,val_11pt_1,val_11pt_2) ;
    Tet11Pt[5]  = Vec3D(val_11pt_3,val_11pt_3,val_11pt_4) ;
    Tet11Pt[6]  = Vec3D(val_11pt_4,val_11pt_3,val_11pt_3) ;
    Tet11Pt[7]  = Vec3D(val_11pt_4,val_11pt_4,val_11pt_3) ;
    Tet11Pt[8]  = Vec3D(val_11pt_3,val_11pt_4,val_11pt_4) ;
    Tet11Pt[9]  = Vec3D(val_11pt_3,val_11pt_4,val_11pt_3) ;
    Tet11Pt[10] = Vec3D(val_11pt_4,val_11pt_3,val_11pt_4) ;

    Tet11Wt = TVector<double>(11) ;
    Tet11Wt[0]  = -0.013155555555555 ;
    Tet11Wt[1]  =  0.007622222222222 ;
    Tet11Wt[2]  =  0.007622222222222 ;
    Tet11Wt[3]  =  0.007622222222222 ;
    Tet11Wt[4]  =  0.007622222222222 ;
    Tet11Wt[5]  =  0.024888888888888 ;
    Tet11Wt[6]  =  0.024888888888888 ;
    Tet11Wt[7]  =  0.024888888888888 ;
    Tet11Wt[8]  =  0.024888888888888 ;
    Tet11Wt[9]  =  0.024888888888888 ;
    Tet11Wt[10] =  0.024888888888888 ;

    // 15 points (degree 5)

    const double val_15pt_1 = 0.333333333333333 ;
    const double val_15pt_2 = 0.090909090909090 ;
    const double val_15pt_3 = 0.727272727272727 ;
    const double val_15pt_4 = 0.066550153573664 ;
    const double val_15pt_5 = 0.433449846426335 ;
 
    Tet15Pt = TVector<Vec3D>(15) ;
    Tet15Pt[0]  = Vec3D(0.25,0.25,0.25) ;
    Tet15Pt[1]  = Vec3D(val_15pt_1,val_15pt_1,val_15pt_1) ;
    Tet15Pt[2]  = Vec3D(       0.0,val_15pt_1,val_15pt_1) ;
    Tet15Pt[3]  = Vec3D(val_15pt_1,       0.0,val_15pt_1) ;
    Tet15Pt[4]  = Vec3D(val_15pt_1,val_15pt_1,       0.0) ;
    Tet15Pt[5]  = Vec3D(val_15pt_2,val_15pt_2,val_15pt_2) ;
    Tet15Pt[6]  = Vec3D(val_15pt_3,val_15pt_2,val_15pt_2) ;
    Tet15Pt[7]  = Vec3D(val_15pt_2,val_15pt_3,val_15pt_2) ;
    Tet15Pt[8]  = Vec3D(val_15pt_2,val_15pt_2,val_15pt_3) ;
    Tet15Pt[9]  = Vec3D(val_15pt_4,val_15pt_4,val_15pt_5) ;
    Tet15Pt[10] = Vec3D(val_15pt_5,val_15pt_4,val_15pt_4) ;
    Tet15Pt[11] = Vec3D(val_15pt_5,val_15pt_5,val_15pt_4) ;
    Tet15Pt[12] = Vec3D(val_15pt_4,val_15pt_5,val_15pt_5) ;
    Tet15Pt[13] = Vec3D(val_15pt_4,val_15pt_5,val_15pt_4) ;
    Tet15Pt[14] = Vec3D(val_15pt_5,val_15pt_4,val_15pt_5) ;

    Tet15Wt = TVector<double>(15) ;
    Tet15Wt[0]  = 0.030283678097089 ;
    Tet15Wt[1]  = 0.006026785714285 ;
    Tet15Wt[2]  = 0.006026785714285 ;
    Tet15Wt[3]  = 0.006026785714285 ;
    Tet15Wt[4]  = 0.006026785714285 ;
    Tet15Wt[5]  = 0.011645249086028 ;
    Tet15Wt[6]  = 0.011645249086028 ;
    Tet15Wt[7]  = 0.011645249086028 ;
    Tet15Wt[8]  = 0.011645249086028 ;
    Tet15Wt[9]  = 0.010949141561386 ;
    Tet15Wt[10] = 0.010949141561386 ;
    Tet15Wt[11] = 0.010949141561386 ;
    Tet15Wt[12] = 0.010949141561386 ;
    Tet15Wt[13] = 0.010949141561386 ;
    Tet15Wt[14] = 0.010949141561386 ;

    // 24 points (degree 6)

    const double val_24pt_1  = 0.214602871259151 ;
    const double val_24pt_2  = 0.356191386222545 ;
    const double val_24pt_3  = 0.040673958534611 ;
    const double val_24pt_4  = 0.877978124396166 ;
    const double val_24pt_5  = 0.322337890142275 ;
    const double val_24pt_6  = 0.032986329573173 ;
    const double val_24pt_7  = 0.063661001875017 ;
    const double val_24pt_8  = 0.269672331458316 ;
    const double val_24pt_9  = 0.603005664479165 ;

 
    Tet24Pt = TVector<Vec3D>(24) ;
    Tet24Pt[0]  = Vec3D(val_24pt_1,val_24pt_1,val_24pt_1) ;
    Tet24Pt[1]  = Vec3D(val_24pt_2,val_24pt_1,val_24pt_1) ;
    Tet24Pt[2]  = Vec3D(val_24pt_1,val_24pt_2,val_24pt_1) ;
    Tet24Pt[3]  = Vec3D(val_24pt_1,val_24pt_1,val_24pt_2) ;
    Tet24Pt[4]  = Vec3D(val_24pt_3,val_24pt_3,val_24pt_3) ;
    Tet24Pt[5]  = Vec3D(val_24pt_4,val_24pt_3,val_24pt_3) ;
    Tet24Pt[6]  = Vec3D(val_24pt_3,val_24pt_4,val_24pt_3) ;
    Tet24Pt[7]  = Vec3D(val_24pt_3,val_24pt_3,val_24pt_4) ;
    Tet24Pt[8]  = Vec3D(val_24pt_5,val_24pt_5,val_24pt_5) ;
    Tet24Pt[9]  = Vec3D(val_24pt_6,val_24pt_5,val_24pt_5) ;
    Tet24Pt[10] = Vec3D(val_24pt_5,val_24pt_6,val_24pt_5) ;
    Tet24Pt[11] = Vec3D(val_24pt_5,val_24pt_5,val_24pt_6) ;
    Tet24Pt[12] = Vec3D(val_24pt_7,val_24pt_7,val_24pt_8) ;
    Tet24Pt[13] = Vec3D(val_24pt_9,val_24pt_7,val_24pt_7) ;
    Tet24Pt[14] = Vec3D(val_24pt_8,val_24pt_9,val_24pt_7) ;
    Tet24Pt[15] = Vec3D(val_24pt_7,val_24pt_8,val_24pt_9) ;
    Tet24Pt[16] = Vec3D(val_24pt_7,val_24pt_8,val_24pt_7) ;
    Tet24Pt[17] = Vec3D(val_24pt_9,val_24pt_7,val_24pt_8) ;
    Tet24Pt[18] = Vec3D(val_24pt_7,val_24pt_9,val_24pt_7) ;
    Tet24Pt[19] = Vec3D(val_24pt_8,val_24pt_7,val_24pt_9) ;
    Tet24Pt[20] = Vec3D(val_24pt_7,val_24pt_9,val_24pt_8) ;
    Tet24Pt[21] = Vec3D(val_24pt_7,val_24pt_7,val_24pt_9) ;
    Tet24Pt[22] = Vec3D(val_24pt_8,val_24pt_7,val_24pt_7) ;
    Tet24Pt[23] = Vec3D(val_24pt_9,val_24pt_8,val_24pt_7) ;

    Tet24Wt = TVector<double>(24) ;
    Tet24Wt[0]  = 0.006653791709695 ;
    Tet24Wt[1]  = 0.006653791709695 ;
    Tet24Wt[2]  = 0.006653791709695 ;
    Tet24Wt[3]  = 0.006653791709695 ;
    Tet24Wt[4]  = 0.001679535175887 ;
    Tet24Wt[5]  = 0.001679535175887 ;
    Tet24Wt[6]  = 0.001679535175887 ;
    Tet24Wt[7]  = 0.001679535175887 ;
    Tet24Wt[8]  = 0.009226196923942 ;
    Tet24Wt[9]  = 0.009226196923942 ;
    Tet24Wt[10] = 0.009226196923942 ;
    Tet24Wt[11] = 0.009226196923942 ;
    Tet24Wt[12] = 0.008035714285714 ;
    Tet24Wt[13] = 0.008035714285714 ;
    Tet24Wt[14] = 0.008035714285714 ;
    Tet24Wt[15] = 0.008035714285714 ;
    Tet24Wt[16] = 0.008035714285714 ;
    Tet24Wt[17] = 0.008035714285714 ;
    Tet24Wt[18] = 0.008035714285714 ;
    Tet24Wt[19] = 0.008035714285714 ;
    Tet24Wt[20] = 0.008035714285714 ;
    Tet24Wt[21] = 0.008035714285714 ;
    Tet24Wt[22] = 0.008035714285714 ;
    Tet24Wt[23] = 0.008035714285714 ;
}

const TVector<Vec3D> &TetIntegrationPts::GetVolumeCoords(int number) const
{
    assert((number == 1) || (number == 4) || (number == 5) ||
           (number == 11) || (number == 15) || (number == 24)) ;
    switch (number) {
        case 1:  return(Tet1Pt) ;
        case 4:  return(Tet4Pt) ;
        case 5:  return(Tet5Pt) ;
        case 11: return(Tet11Pt) ;
        case 15: return(Tet15Pt) ;
        case 24: return(Tet24Pt) ;
    }
    return(NullPt) ;
}

const TVector<double> &TetIntegrationPts::GetVolumeWeights(int number) const
{
    assert((number == 1) || (number == 4) || (number == 5) ||
           (number == 11) || (number == 15) || (number == 24)) ;
    switch (number) {
        case 1:  return(Tet1Wt) ;
        case 4:  return(Tet4Wt) ;
        case 5:  return(Tet5Wt) ;
        case 11: return(Tet11Wt) ;
        case 15: return(Tet15Wt) ;
        case 24: return(Tet24Wt) ;
    }
    return(NullWt) ;
}

// ------------------------------------------------------
//  Implementation for the BrickIntegration class

BrickIntegrationPts *BrickIntegrationPts::_instance = 0 ;

BrickIntegrationPts *BrickIntegrationPts::Instance()
{
    if (_instance == 0) {
        _instance = new BrickIntegrationPts ;
    }
    return _instance ;
}

BrickIntegrationPts::BrickIntegrationPts() : IntegrationPts()
{
    // 1 point

    Brick1Pt = TVector<Vec3D>(1) ;
    Brick1Pt[0] = Vec3D(0.0,0.0,0.0) ;

    Brick1Wt = TVector<double>(1) ;
    Brick1Wt[0] = 8.0 ;

    // 8 points

    const double val_8pt = 0.577350269189626 ;
 
    Brick8Pt = TVector<Vec3D>(8) ;
    Brick8Pt[0] = Vec3D(-val_8pt,-val_8pt,-val_8pt) ;
    Brick8Pt[1] = Vec3D( val_8pt,-val_8pt,-val_8pt) ;
    Brick8Pt[2] = Vec3D(-val_8pt, val_8pt,-val_8pt) ;
    Brick8Pt[3] = Vec3D( val_8pt, val_8pt,-val_8pt) ;
    Brick8Pt[4] = Vec3D(-val_8pt,-val_8pt, val_8pt) ;
    Brick8Pt[5] = Vec3D( val_8pt,-val_8pt, val_8pt) ;
    Brick8Pt[6] = Vec3D(-val_8pt, val_8pt, val_8pt) ;
    Brick8Pt[7] = Vec3D( val_8pt, val_8pt, val_8pt) ;

    Brick8Wt = TVector<double>(8) ;
    for (int i=0 ; i<8 ; ++i) Brick8Wt[i] = 1.0 ;

    // 27 points

    const double val_27pt = 0.774596669241483 ;
 
    Brick27Pt = TVector<Vec3D>(27) ;
    Brick27Pt[ 0] = Vec3D(-val_27pt,-val_27pt,-val_27pt) ;
    Brick27Pt[ 1] = Vec3D(      0.0,-val_27pt,-val_27pt) ;
    Brick27Pt[ 2] = Vec3D( val_27pt,-val_27pt,-val_27pt) ;
    Brick27Pt[ 3] = Vec3D(-val_27pt,      0.0,-val_27pt) ;
    Brick27Pt[ 4] = Vec3D(      0.0,      0.0,-val_27pt) ;
    Brick27Pt[ 5] = Vec3D( val_27pt,      0.0,-val_27pt) ;
    Brick27Pt[ 6] = Vec3D(-val_27pt, val_27pt,-val_27pt) ;
    Brick27Pt[ 7] = Vec3D(      0.0, val_27pt,-val_27pt) ;
    Brick27Pt[ 8] = Vec3D( val_27pt, val_27pt,-val_27pt) ;
    Brick27Pt[ 9] = Vec3D(-val_27pt,-val_27pt,      0.0) ;
    Brick27Pt[10] = Vec3D(      0.0,-val_27pt,      0.0) ;
    Brick27Pt[11] = Vec3D( val_27pt,-val_27pt,      0.0) ;
    Brick27Pt[12] = Vec3D(-val_27pt,      0.0,      0.0) ;
    Brick27Pt[13] = Vec3D(      0.0,      0.0,      0.0) ;
    Brick27Pt[14] = Vec3D( val_27pt,      0.0,      0.0) ;
    Brick27Pt[15] = Vec3D(-val_27pt, val_27pt,      0.0) ;
    Brick27Pt[16] = Vec3D(      0.0, val_27pt,      0.0) ;
    Brick27Pt[17] = Vec3D( val_27pt, val_27pt,      0.0) ;
    Brick27Pt[18] = Vec3D(-val_27pt,-val_27pt, val_27pt) ;
    Brick27Pt[19] = Vec3D(      0.0,-val_27pt, val_27pt) ;
    Brick27Pt[20] = Vec3D( val_27pt,-val_27pt, val_27pt) ;
    Brick27Pt[21] = Vec3D(-val_27pt,      0.0, val_27pt) ;
    Brick27Pt[22] = Vec3D(      0.0,      0.0, val_27pt) ;
    Brick27Pt[23] = Vec3D( val_27pt,      0.0, val_27pt) ;
    Brick27Pt[24] = Vec3D(-val_27pt, val_27pt, val_27pt) ;
    Brick27Pt[25] = Vec3D(      0.0, val_27pt, val_27pt) ;
    Brick27Pt[26] = Vec3D( val_27pt, val_27pt, val_27pt) ;

    Brick27Wt = TVector<double>(27) ;
    Brick27Wt[ 0] = 0.171467764 ;
    Brick27Wt[ 1] = 0.274348422 ;
    Brick27Wt[ 2] = 0.171467764 ;
    Brick27Wt[ 3] = 0.274348422 ;
    Brick27Wt[ 4] = 0.438957476 ;
    Brick27Wt[ 5] = 0.274348422 ;
    Brick27Wt[ 6] = 0.171467764 ;
    Brick27Wt[ 7] = 0.274348422 ;
    Brick27Wt[ 8] = 0.171467764 ;
    Brick27Wt[ 9] = 0.274348422 ;
    Brick27Wt[10] = 0.438957476 ;
    Brick27Wt[11] = 0.274348422 ;
    Brick27Wt[12] = 0.438957476 ;
    Brick27Wt[13] = 0.702331962 ;
    Brick27Wt[14] = 0.438957476 ;
    Brick27Wt[15] = 0.274348422 ;
    Brick27Wt[16] = 0.438957476 ;
    Brick27Wt[17] = 0.274348422 ;
    Brick27Wt[18] = 0.171467764 ;
    Brick27Wt[19] = 0.274348422 ;
    Brick27Wt[20] = 0.171467764 ;
    Brick27Wt[21] = 0.274348422 ;
    Brick27Wt[22] = 0.438957476 ;
    Brick27Wt[23] = 0.274348422 ;
    Brick27Wt[24] = 0.171467764 ;
    Brick27Wt[25] = 0.274348422 ;
    Brick27Wt[26] = 0.171467764 ;

    // 64 points

    const double val_64pt[4] = {
            -0.861136311594953,-0.339981043584856,
             0.339981043584856, 0.861136311594953 } ;
 
    Brick64Pt = TVector<Vec3D>(64) ;
    int cur = 0 ;
    for (int i=0 ; i<4 ; ++i) {
        for (int j=0 ; j<4 ; ++j) {
            for (int k=0 ; k<4 ; ++k) {
                Brick64Pt[cur] =
                     Vec3D(val_64pt[i],val_64pt[j],val_64pt[k]) ;
                ++cur ;
            }
        }
    }

    const double val_64wt[4] = {
             0.347854845137454, 0.652145154862546,
             0.652145154862546, 0.347854845137454 } ;

    Brick64Wt = TVector<double>(64) ;
    cur = 0 ;
    for (int i=0 ; i<4 ; ++i) {
        for (int j=0 ; j<4 ; ++j) {
            for (int k=0 ; k<4 ; ++k) {
                Brick64Wt[cur] = val_64wt[i]*val_64wt[j]*val_64wt[k] ;
                ++cur ;
            }
        }
    }
}

const TVector<Vec3D> &BrickIntegrationPts::GetVolumeCoords(int number) const
{
    assert((number == 1) || (number == 8) || (number == 27) ||
           (number == 64)) ;
    switch (number) {
        case  1: return(Brick1Pt) ;
        case  8: return(Brick8Pt) ;
        case 27: return(Brick27Pt) ;
        case 64: return(Brick64Pt) ;
    }
    return(NullPt) ;
}

const TVector<double> &BrickIntegrationPts::GetVolumeWeights(int number) const
{
    assert((number == 1) || (number == 8) || (number == 27) ||
           (number == 64)) ;
    switch (number) {
        case  1: return(Brick1Wt) ;
        case  8: return(Brick8Wt) ;
        case 27: return(Brick27Wt) ;
        case 64: return(Brick64Wt) ;
    }
    return(NullWt) ;
}

// ------------------------------------------------------
//  Implementation for the WedgeIntegration class

WedgeIntegrationPts *WedgeIntegrationPts::_instance = 0 ;

WedgeIntegrationPts *WedgeIntegrationPts::Instance()
{
    if (_instance == 0) {
        _instance = new WedgeIntegrationPts ;
    }
    return _instance ;
}

WedgeIntegrationPts::WedgeIntegrationPts() : IntegrationPts()
{
    // 6 points

    const double val_6pt = 0.577350269189626 ;
 
    Wedge6Pt = TVector<Vec3D>(6) ;
    Wedge6Pt[0] = Vec3D(0.5,0.5,-val_6pt) ;
    Wedge6Pt[1] = Vec3D(0.0,0.5,-val_6pt) ;
    Wedge6Pt[2] = Vec3D(0.5,0.0,-val_6pt) ;
    Wedge6Pt[3] = Vec3D(0.5,0.5,-val_6pt) ;
    Wedge6Pt[4] = Vec3D(0.0,0.5, val_6pt) ;
    Wedge6Pt[5] = Vec3D(0.5,0.0, val_6pt) ;

    Wedge6Wt = TVector<double>(6) ;
    for (int i=0 ; i<6 ; ++i) Wedge6Wt[i] = 0.166666666667 ;

    // 12 points

    const double val_12pt = 0.774596669241483 ;
 
    Wedge12Pt = TVector<Vec3D>(12) ;
    Wedge12Pt[ 0] = Vec3D(0.333333333,0.333333333,-val_12pt) ;
    Wedge12Pt[ 1] = Vec3D(0.6,0.2,-val_12pt) ;
    Wedge12Pt[ 2] = Vec3D(0.2,0.6,-val_12pt) ;
    Wedge12Pt[ 3] = Vec3D(0.2,0.2,-val_12pt) ;
    Wedge12Pt[ 4] = Vec3D(0.333333333,0.333333333,0.0) ;
    Wedge12Pt[ 5] = Vec3D(0.6,0.2,0.0) ;
    Wedge12Pt[ 6] = Vec3D(0.2,0.6,0.0) ;
    Wedge12Pt[ 7] = Vec3D(0.2,0.2,0.0) ;
    Wedge12Pt[ 8] = Vec3D(0.333333333,0.333333333,val_12pt) ;
    Wedge12Pt[ 9] = Vec3D(0.6,0.2,val_12pt) ;
    Wedge12Pt[10] = Vec3D(0.2,0.6,val_12pt) ;
    Wedge12Pt[11] = Vec3D(0.2,0.2,val_12pt) ;

    Wedge12Wt = TVector<double>(12) ;
    Wedge12Wt[ 0] = -0.15625 ;
    Wedge12Wt[ 1] =  0.144675926 ;
    Wedge12Wt[ 2] =  0.144675926 ;
    Wedge12Wt[ 3] =  0.144675926 ;
    Wedge12Wt[ 4] = -0.25 ;
    Wedge12Wt[ 5] =  0.231481482 ;
    Wedge12Wt[ 6] =  0.231481482 ;
    Wedge12Wt[ 7] =  0.231481482 ;
    Wedge12Wt[ 8] = -0.15625 ;
    Wedge12Wt[ 9] =  0.144675926 ;
    Wedge12Wt[10] =  0.144675926 ;
    Wedge12Wt[11] =  0.144675926 ;

    // 28 points

    const double val_7pt1 = 0.0597158717 ;
    const double val_7pt2 = 0.4701420641 ;
    const double val_7pt3 = 0.7974269853 ;
    const double val_7pt4 = 0.1012865073 ;
    const double val_28pta = 0.861136311595 ;
    const double val_28ptb = 0.339981043585 ;

    Wedge28Pt = TVector<Vec3D>(28) ;
    Wedge28Pt[ 0] = Vec3D(0.333333333,0.333333333,-val_28pta) ;
    Wedge28Pt[ 1] = Vec3D(val_7pt1,val_7pt2,-val_28pta) ;
    Wedge28Pt[ 2] = Vec3D(val_7pt2,val_7pt1,-val_28pta) ;
    Wedge28Pt[ 3] = Vec3D(val_7pt2,val_7pt2,-val_28pta) ;
    Wedge28Pt[ 4] = Vec3D(val_7pt3,val_7pt4,-val_28pta) ;
    Wedge28Pt[ 5] = Vec3D(val_7pt4,val_7pt3,-val_28pta) ;
    Wedge28Pt[ 6] = Vec3D(val_7pt4,val_7pt4,-val_28pta) ;
    Wedge28Pt[ 7] = Vec3D(0.333333333,0.333333333,-val_28ptb) ;
    Wedge28Pt[ 8] = Vec3D(val_7pt1,val_7pt2,-val_28ptb) ;
    Wedge28Pt[ 9] = Vec3D(val_7pt2,val_7pt1,-val_28ptb) ;
    Wedge28Pt[10] = Vec3D(val_7pt2,val_7pt2,-val_28ptb) ;
    Wedge28Pt[11] = Vec3D(val_7pt3,val_7pt4,-val_28ptb) ;
    Wedge28Pt[12] = Vec3D(val_7pt4,val_7pt3,-val_28ptb) ;
    Wedge28Pt[13] = Vec3D(val_7pt4,val_7pt4,-val_28ptb) ;
    Wedge28Pt[14] = Vec3D(0.333333333,0.333333333,val_28ptb) ;
    Wedge28Pt[15] = Vec3D(val_7pt1,val_7pt2,val_28ptb) ;
    Wedge28Pt[16] = Vec3D(val_7pt2,val_7pt1,val_28ptb) ;
    Wedge28Pt[17] = Vec3D(val_7pt2,val_7pt2,val_28ptb) ;
    Wedge28Pt[18] = Vec3D(val_7pt3,val_7pt4,val_28ptb) ;
    Wedge28Pt[19] = Vec3D(val_7pt4,val_7pt3,val_28ptb) ;
    Wedge28Pt[20] = Vec3D(val_7pt4,val_7pt4,val_28ptb) ;
    Wedge28Pt[21] = Vec3D(0.333333333,0.333333333,val_28pta) ;
    Wedge28Pt[22] = Vec3D(val_7pt1,val_7pt2,val_28pta) ;
    Wedge28Pt[23] = Vec3D(val_7pt2,val_7pt1,val_28pta) ;
    Wedge28Pt[24] = Vec3D(val_7pt2,val_7pt2,val_28pta) ;
    Wedge28Pt[25] = Vec3D(val_7pt3,val_7pt4,val_28pta) ;
    Wedge28Pt[26] = Vec3D(val_7pt4,val_7pt3,val_28pta) ;
    Wedge28Pt[27] = Vec3D(val_7pt4,val_7pt4,val_28pta) ;

    Wedge28Wt = TVector<double>(28) ;
    Wedge28Wt[ 0] = 0.039133670 ;
    Wedge28Wt[ 1] = 0.023026974 ;
    Wedge28Wt[ 2] = 0.023026974 ;
    Wedge28Wt[ 3] = 0.023026974 ;
    Wedge28Wt[ 4] = 0.021904277 ;
    Wedge28Wt[ 5] = 0.021904277 ;
    Wedge28Wt[ 6] = 0.021904277 ;
    Wedge28Wt[ 7] = 0.073366330 ;
    Wedge28Wt[ 8] = 0.043170103 ;
    Wedge28Wt[ 9] = 0.043170103 ;
    Wedge28Wt[10] = 0.043170103 ;
    Wedge28Wt[11] = 0.041065313 ;
    Wedge28Wt[12] = 0.041065313 ;
    Wedge28Wt[13] = 0.041065313 ;
    Wedge28Wt[14] = 0.073366330 ;
    Wedge28Wt[15] = 0.043170103 ;
    Wedge28Wt[16] = 0.043170103 ;
    Wedge28Wt[17] = 0.043170103 ;
    Wedge28Wt[18] = 0.041065313 ;
    Wedge28Wt[19] = 0.041065313 ;
    Wedge28Wt[20] = 0.041065313 ;
    Wedge28Wt[21] = 0.039133670 ;
    Wedge28Wt[22] = 0.023026974 ;
    Wedge28Wt[23] = 0.023026974 ;
    Wedge28Wt[24] = 0.023026974 ;
    Wedge28Wt[25] = 0.021904277 ;
    Wedge28Wt[26] = 0.021904277 ;
    Wedge28Wt[27] = 0.021904277 ;

    // 48 points

    const double val_12pt1 = 0.8738219710 ;
    const double val_12pt2 = 0.0630890144 ;
    const double val_12pt3 = 0.5014265096 ;
    const double val_12pt4 = 0.2492867451 ;
    const double val_12pt5 = 0.6365024991 ;
    const double val_12pt6 = 0.3103524510 ;
    const double val_12pt7 = 0.0531450498 ;

    const double tri_12pt[12][2] = 
        {{val_12pt1,val_12pt2},{val_12pt2,val_12pt2},
         {val_12pt2,val_12pt1},{val_12pt3,val_12pt4},
         {val_12pt4,val_12pt4},{val_12pt4,val_12pt3},
         {val_12pt5,val_12pt6},{val_12pt5,val_12pt7},
         {val_12pt6,val_12pt7},{val_12pt6,val_12pt5},
         {val_12pt7,val_12pt5},{val_12pt7,val_12pt6}} ;

    const double val_12wt1 = 0.025422453 ;
    const double val_12wt2 = 0.058393138 ;
    const double val_12wt3 = 0.041425538 ;

    const double tri_12wt[12] =
      {val_12wt1,val_12wt1,val_12wt1,val_12wt2,val_12wt2,val_12wt2,
       val_12wt3,val_12wt3,val_12wt3,val_12wt3,val_12wt3,val_12wt3} ;

    const double lin_4pt[4] = {
            -0.861136311594953,-0.339981043584856,
             0.339981043584856, 0.861136311594953 } ;
 
    const double lin_4wt[4] = {
             0.347854845137454, 0.652145154862546,
             0.652145154862546, 0.347854845137454 } ;

    Wedge48Pt = TVector<Vec3D>(48) ;
    int cur = 0 ;
    for (int i=0 ; i<4 ; ++i) {
        for (int j=0 ; j<12 ; ++j) {
            Wedge48Pt[cur] =
                     Vec3D(tri_12pt[j][0],tri_12pt[j][1],lin_4pt[i]) ;
            ++cur ;
        }
    }

    Wedge48Wt = TVector<double>(48) ;
    cur = 0 ;
    for (int i=0 ; i<4 ; ++i) {
        for (int j=0 ; j<12 ; ++j) {
            Wedge48Wt[cur] = tri_12wt[j]*lin_4wt[i] ;
            ++cur ;
        }
    }
}

const TVector<Vec3D> &WedgeIntegrationPts::GetVolumeCoords(int number) const
{
    assert((number == 6) || (number == 12) || (number == 28) ||
           (number == 48)) ;
    switch (number) {
        case  6: return(Wedge6Pt) ;
        case 12: return(Wedge12Pt) ;
        case 28: return(Wedge28Pt) ;
        case 48: return(Wedge48Pt) ;
    }
    return(NullPt) ;
}

const TVector<double> &WedgeIntegrationPts::GetVolumeWeights(int number) const
{
    assert((number == 6) || (number == 12) || (number == 28) ||
           (number == 48)) ;
    switch (number) {
        case  6: return(Wedge6Wt) ;
        case 12: return(Wedge12Wt) ;
        case 28: return(Wedge28Wt) ;
        case 48: return(Wedge48Wt) ;
    }
    return(NullWt) ;
}

} // namespace
