
//
// Jacobian Matrix class
//
// Revision -
//   $Revision: 1.1.1.1 $  $Date: 2001/12/02 16:53:08 $  $Author: wash $
//

#ifndef Jacobian_hh
#define Jacobian_hh

#include <cassert>
#include <iostream>

namespace FemLib {

// Class Jacobian

class Jacobian : public TMatrix<double> {
    public:

        Jacobian() : TMatrix<double>(3,3) {} ;

        Jacobian(const TMatrix<double> &oper1,
                 const TVector<Vec3D> &oper2,
                 int num_pts = -1) : TMatrix<double>(3,3)
        {
            assert(oper1.Rows() == 3) ;
            int num = num_pts < 0 ? oper1.Cols() : num_pts ;

            for (int i=0 ; i<3 ; ++i) {
                for (int j=0 ; j<3 ; ++j) {
                    data[i*3+j] = 0 ;
                    for (int k=0 ; k<num ; ++k) {
                        data[i*3+j] += oper1(i,k) * oper2(k)[j] ;
                    }
                }
            }
        } ;

        double Determinate()
        {
            double det = data[0]*data[4]*data[8] +
                         data[1]*data[5]*data[6] +
                         data[2]*data[3]*data[7] -
                         data[6]*data[4]*data[2] -
                         data[7]*data[5]*data[0] -
                         data[8]*data[3]*data[1] ;
            return(det) ;
        } ;

        Jacobian Inverse(double det)
        {
            Jacobian inv ;
            inv(0,0) =  (data[4]*data[8] - data[5]*data[7]) / det ;
            inv(0,1) = -(data[1]*data[8] - data[2]*data[7]) / det ;
            inv(0,2) =  (data[1]*data[5] - data[2]*data[4]) / det ;
            inv(1,0) = -(data[3]*data[8] - data[5]*data[6]) / det ;
            inv(1,1) =  (data[0]*data[8] - data[2]*data[6]) / det ;
            inv(1,2) = -(data[0]*data[5] - data[2]*data[3]) / det ;
            inv(2,0) =  (data[3]*data[7] - data[4]*data[6]) / det ;
            inv(2,1) = -(data[0]*data[7] - data[1]*data[6]) / det ;
            inv(2,2) =  (data[0]*data[4] - data[1]*data[3]) / det ;
            return(inv) ;
        } ;


    private:
} ;

class SurfaceJacobian : public TMatrix<double> {
    public:

        SurfaceJacobian() : TMatrix<double>(3,3) {} ;

        SurfaceJacobian(const TMatrix<double> &oper1,
                        const TVector<Vec3D> &oper2,
                        int num_pts = -1) : TMatrix<double>(3,3)
        {
            assert(oper1.Rows() == 2) ;
            int num = num_pts < 0 ? oper1.Cols() : num_pts ;

            for (int i=0 ; i<num ; ++i) {
                data[0] += oper1(0,i) * oper2(i).x() ;
                data[1] += oper1(0,i) * oper2(i).y() ;
                data[2] += oper1(0,i) * oper2(i).z() ;
                data[3] += oper1(1,i) * oper2(i).x() ;
                data[4] += oper1(1,i) * oper2(i).y() ;
                data[5] += oper1(1,i) * oper2(i).z() ;
            }

            // the third row is the cross product of the first to rows

            data[6] = data[1]*data[5] - data[4]*data[2] ;
            data[7] = data[2]*data[3] - data[5]*data[0] ;
            data[8] = data[0]*data[4] - data[3]*data[1] ;
        } ;

        double Determinate()
        {
            // the determinate is the length of the third row

            return(sqrt(data[6]*data[6] +
                        data[7]*data[7] +
                        data[8]*data[8])) ;
        } ;

    private:
} ;

} // namespace

#endif
