//
// Material Matrix class header file
//
// Revision -
//   $Revision: 1.1.1.1 $  $Date: 2001/12/02 16:53:08 $  $Author: wash $
//
 
#ifndef MaterialMat_h
#define MaterialMat_h

#include "TMatrix.hh"

namespace FemLib {

class MaterialMat {

    public:
        static MaterialMat *Instance() ;

        TMatrix<double> IsotropicMatMatrix(double E,double nu) ;

        TMatrix<double> OrthotropicMatMatrixFromEngngConstants(double E1,
                    double E2,double E3,double nu12,double nu23,
                    double nu31,double G12,double G23,double G31,
                    double angles[3]) ;

        TMatrix<double> OrthotropicMatMatrixFromStiffnessComps(
                    double c[9],double angle[3]) ;

        TMatrix<double> OrthotropicMatMatrixFromComplianceComps(
                    double s[9],double angle[3]) ;

    protected:
        MaterialMat() {} ;

    private:
        static MaterialMat *_instance ;

        TMatrix<double> RotateOrthoMatrix(TMatrix<double> Dmtx,
                                          double angles[3]) ;
} ;

/*

For all the orthotropic methods the angles and rotations are
defined as follows:

    angle[0] - first rotation about the z axis (positive x axis to y)
    angle[1] - second rotation about the (intermediate local) y axis
               (positive z axis to x)
    angle[2] - third rotation about (intermediate local) y axis
               (positive z axis to x)

    Angles are given in DEGREES.


TMatrix<double> OrthotropicMatMatrixFromEngngConstants(double E1,
                    double E2,double E3,double nu12,double nu23,
                    double nu12,double G12,double G23,double G31,
                    double angles[3]) ;

Formulates an orthotropic constitutive matrix from engineering
constants.


TMatrix<double> OrthotropicMatMatrixFromStiffnessComps(
                    double c[9],double angle[3]) ;

Formulates an orthotropic constitutive matrix from the the given
matrix components.  They are defined as:

    c[0] - c11
    c[1] - c22
    c[2] - c33
    c[3] - c12 = c21
    c[4] - c23 = c32
    c[5] - c13 = c31
    c[6] - c44  (Gxy)
    c[7] - c55  (Gyz)
    c[8] - c66  (Gzx)


TMatrix<double> OrthotropicMatMatrixFromComplianceComps(
                    double s[9],double angle[3]) ;

Formulates an orthotropic constitutive matrix from the given
compliance components, defined as:

    s[0] - s11 = 1.0/E1
    s[1] - s22 = 1.0/E2
    s[2] - s33 = 1.0/E3
    s[3] - s21 = s12 = -nu21/E2 = -nu12/E1
    s[4] - s32 = s23 = -nu32/E3 = -nu23/E2
    s[5] - s31 = s13 = -nu31/E1 = -nu13/E3
    s[6] - s44 = 1.0/G12
    s[7] - s55 = 1.0/G23
    s[8] - s66 = 1.0/G31
*/

class ScalarMaterialMat {

    public:
        static ScalarMaterialMat *Instance() ;

        TMatrix<double> IsotropicMatMatrix(double k) ;

        TMatrix<double> OrthotropicMatMatrix(double k1,
                    double k2,double k3,double angles[3]) ;

    protected:
        ScalarMaterialMat() {} ;

    private:
        static ScalarMaterialMat *_instance ;

        TMatrix<double> RotateOrthoMatrix(TMatrix<double> Dmtx,
                                          double angles[3]) ;
} ;

} // namespace

#endif
