#!/usr/local/bin/python

import sys
import string

from Vec3D import Vec3D
from Numeric import *
from LinearAlgebra import *
from pprint import pprint
sys.path.append("../MDWebServices/Utilities")
#from Utilities import *


class SimpleFemModel:
    """Base class for other FemModel classes which act as a container
       to hold a description of a mesh and associated elements and
       boundary conditions."""

    # constructor does nothing - overridded by derived classes

    def __init__(self):
        print "constructor for base class does nothing"


    def __repr__(self):
        # function called when object passed to print
        # currently does nothing
	reprString = "MODEL DATA:\n"
	reprString = reprString + "blah!\n"

	return reprString



    # Functions used by other FemModel classes

    def OrderedPair(self,pair):
        if pair[0] > pair[1]:
            orderedPair = (pair[1],pair[0])
        else:
            orderedPair = pair
        return orderedPair

    def GetEdgeNode(self, pair):
        # the dictionary which maps pairs of corner nodes to
        # edge nodes assumes that the pairs are ordered
        orderedPair = self.OrderedPair(pair)

        # if this pair is already in the dictionary then we just look up the
        # value
        if self.edgeNodeMap.has_key(orderedPair):
            return self.edgeNodeMap[orderedPair]
        else:
            endVert0 = self.nodes[orderedPair[0]]
            endVert1 = self.nodes[orderedPair[1]]
            newEdgeNodeID = self.GetNewNodeID()
            newEdgeNodePos = (0.5 * (endVert0[0] + endVert1[0]), 0.5 * (endVert0[1] + endVert1[1]),0.5 * (endVert0[2] + endVert1[2]))
            
            self.nodes[newEdgeNodeID] = newEdgeNodePos
            self.edgeNodeMap[orderedPair] = newEdgeNodeID
            return newEdgeNodeID
        

    def GetElemCoords(self,i,vec3d=1):
        """Return a list of nodal coordinates for element i"""
        coords = []
        for node in self.elem[i][2]:
            vals = self.GetNodeCoord(node,vec3d)
            coords.append(vals)
        return coords

    def GetNodeCoord(self,id,vec3d=1):
        if vec3d:
            return Vec3D(self.nodes[id][0],self.nodes[id][1],self.nodes[id][2])
        else:
            return (self.nodes[id][0],self.nodes[id][1],self.nodes[id][2])

    def NumNodes(self):
        return len(self.nodes.keys())
    
    def NumElems(self):
        return len(self.elem.keys())

    def NumMats(self):
        return len(self.materials.keys())

    def NumFaceTriangles(self):
	return len(self.faceTriangles.keys())

    def NumBCs(self):
	return len(self.faceDisplacements.keys())

    def GetElemData(self,i):
        return self.elem[i]

    def GetMaterialData(self,i):
        return self.materials[i]

    def GetNodeIds(self):
        return self.nodes.keys()

    def GetElemIds(self):
        return self.elem.keys()

    def GetMaterialIds(self):
        return self.materials.keys()

    def GetFaceIds(self):
        return self.faceTriangles.keys()

    def GetFaceTriangles(self,id):
        return self.faceTriangles[id]

    def GetDispBCs(self,id):
        return self.faceDisplacements[id]

    def GetNewNodeID(self):
        ID = len(self.nodes.keys()) + 1
        while 1:
            if ID in self.nodes.keys():
                ID = ID + 1
            else:
                return ID
