
//
// TVector Templated Vector class
//
// Revision -
//   $Revision: 1.1.1.1 $  $Date: 2001/12/02 16:53:08 $  $Author: wash $
//

#ifndef TVector_hh
#define TVector_hh

#include <cassert>
#include <iostream>

#include "TMatrix.hh"

namespace FemLib {

// Class TVector

template<class Dtype> class TVector {
    public:

        // constructors and destructors

        TVector() { rows = 0 ; data = 0 ; } ;

        TVector(const int irows)
        {
            rows = irows ;
            data = new Dtype[rows] ;
            for (int i=0 ; i<rows ; ++i) data[i] = 0 ;
        } ;

        TVector(const TVector &other)
        {
            rows = other.rows ;
            data = new Dtype[rows] ;
            for (int i=0 ; i<rows ; ++i) data[i] = other.data[i] ;
        } ;

        TVector operator = (const TVector &other)
        {
            if (data != 0) delete [] data ;
            rows = other.rows ;
            data = new Dtype[rows] ;
            for (int i=0 ; i<rows ; i++) data[i] = other.data[i] ;
            return(*this) ;
        } ;

        ~TVector() { if (data != 0) delete [] data ; } ;

        // access functions and operators

        int Rows() const { return(rows) ; } ;

        Dtype &operator() (int i)
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        Dtype operator() (int i) const
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        Dtype &operator[] (int i)
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        Dtype operator[] (int i) const
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        Dtype &at(int i)
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        Dtype at(int i) const
        {
            assert(i<rows) ;
            return(data[i]) ;
        } ;

        // computational operators

        TVector &operator += (const Dtype &oper)
        {
            for (int i=0 ; i<rows ; ++i) data[i] += oper ;
            return(*this) ;
        }

        TVector &operator += (const TVector<Dtype> &oper)
        {
            for (int i=0 ; i<rows ; ++i) data[i] += oper.data[i] ;
            return(*this) ;
        }

        TVector &operator *= (Dtype oper) 
        {
            for (int i=0 ; i<rows ; ++i) data[i] *= oper ;
            return(*this) ;
        }

    private:

        int    rows ;
        Dtype *data ;

        // private constructor

        TVector(int irows,Dtype *idata) : rows(irows),data(idata) {} ;

        // friends

#ifdef WIN32
    friend Dtype operator* (const TVector<Dtype> &oper1,
                            const TVector<Dtype> &oper2) ;

    friend TVector<Dtype> operator* (const Dtype &oper1,
                                     const TVector<Dtype> &oper2) ;

    friend TVector<Dtype> operator* (const TVector<Dtype> &oper1,
                                     const Dtype &oper2) ;

    friend TVector<Dtype> operator* (const TMatrix<Dtype> &oper1,
                                     const TVector<Dtype> &oper2) ;
#else
    friend Dtype operator* <Dtype> (const TVector<Dtype> &oper1,
                                    const TVector<Dtype> &oper2) ;

    friend TVector<Dtype> operator* <Dtype> (const Dtype &oper1,
                                             const TVector<Dtype> &oper2) ;

    friend TVector<Dtype> operator* <Dtype> (const TVector<Dtype> &oper1,
                                             const Dtype &oper2) ;

    friend TVector<Dtype> operator* <Dtype> (const TMatrix<Dtype> &oper1,
                                             const TVector<Dtype> &oper2) ;
#endif
} ;


template<class Dtype>
inline Dtype operator *(const TVector<Dtype> &oper1,
                        const TVector<Dtype> &oper2)
{
    assert(oper1.Rows() == oper2.Rows()) ;
    Dtype tmp = 0 ;
    for (int i=0 ; i<oper1.Rows() ; ++i) {
        tmp += oper1(i) * oper2(i) ;
    }
    return(tmp) ;
}


template<class Dtype>
inline TVector<Dtype> operator *(const Dtype &oper1,
                                 const TVector<Dtype> &oper2)
{
    Dtype *tmp = new Dtype[oper2.Rows()] ;

    for (int i=0 ; i<oper2.Rows() ; ++i) {
        tmp[i] = oper1 * oper2(i) ;
    }
    return(TVector<Dtype>(oper2.Rows(),tmp)) ;
}


template<class Dtype>
inline TVector<Dtype> operator *(const TVector<Dtype> &oper1,
                                 const Dtype &oper2)
{
    return(oper2*oper1) ;
}

template<class Dtype>
inline TVector<Dtype> operator *(const TMatrix<Dtype> &oper1,
                                 const TVector<Dtype> &oper2)
{
    assert(oper1.Cols() == oper2.Rows()) ;
    Dtype *tmp = new Dtype[oper1.Rows()] ;

    for (int i=0 ; i<oper1.Rows() ; ++i) {
        tmp[i] = 0 ;
        for (int j=0 ; j<oper1.Cols() ; ++j) {
            tmp[i] += oper1(i,j) * oper2.data[j] ;
        }
    }
    return(TVector<Dtype>(oper1.Rows(),tmp)) ;
}

#if 0
template<class Dtype> inline ostream &operator << (ostream &out,
                                      const TVector<Dtype> &vec)
{
    for (int i=0 ; i<vec.Rows() ; ++i) {
        out << vec(i) << ' ' ;
    }
    out << endl ; 
    return(out) ;
}
#endif

} // namespace

#endif
