
//
// TableFunction header file
//
// Revision -
//   $Revision: 1.1 $  $Date: 2001/12/04 16:13:07 $  $Author: wash $
//

#ifndef TableFunction_h
#define TableFunction_h

class TableFunction {
    public:

        enum InterpolationType {I_LINEAR, I_SPLINE} ;
        enum ExtrapolationType {E_NONE, E_CONSTANT, E_LINEAR} ;

        struct Data {
            double x, y ;
        } ;

        TableFunction() ;

        TableFunction(int num_points,
                      const double *x_values,
                      const double *y_values,
                      InterpolationType interp = I_SPLINE,
                      ExtrapolationType extrap = E_NONE) ;

        TableFunction(const TableFunction &other) ;

        TableFunction operator = (const TableFunction &other) ;

        ~TableFunction() ;

        void Configure(int num_points,
                       const double *x_values,
                       const double *y_values,
                       InterpolationType interp = I_SPLINE,
                       ExtrapolationType extrap = E_NONE) ;

        bool GetValue(double x,double *y) ;

    private:

        struct HermiteCoef {
            double a0, a1, a2, a3 ;
        } ;

        int NumSegments ;
        int CurrentSeg ;
        Data *Vals ;

        HermiteCoef *XCoefs ;
        HermiteCoef *YCoefs ;

        InterpolationType InterpType ;
        ExtrapolationType ExtrapType ;

        double DerivTolerance ;
        double StopTolerance ;
        int MaxIterations ;

        bool ExtrapolateLow(double x,double *y) const ;
        bool ExtrapolateHigh(double x,double *y) const ;

        double HermiteEvaluate(HermiteCoef &coefs,double t) const ;
        double HermiteDerivEvaluate(HermiteCoef &coefs,double t) const ;
} ;

#endif

