
"""Unit test for a TableFunction"""

import TableFunction
import unittest

class ZeroDataPoints(unittest.TestCase):
    def testZeroDataPoints(self):
        """zero data points should always return None"""
        tf = TableFunction.TableFunction([],[])
        y = tf.GetValue(1.0)
        self.assertEqual(y,None)

class OneDataPoints(unittest.TestCase):
    def testOneDataPoints(self):
        """one data point tests"""

        tf = TableFunction.TableFunction([1.0],[10.0],"LINEAR","NONE")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,None)

        x = 1.0
        y = tf.GetValue(x)
        self.assertEqual(y,None)

        x = 1.5
        y = tf.GetValue(x)
        self.assertEqual(y,None)

        tf.Configure([1.0],[10.0],"LINEAR","CONSTANT")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        tf.Configure([1.0],[10.0],"LINEAR","LINEAR")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

class TwoDataPoints(unittest.TestCase):
    def testTwoDataPoints(self):
        """two data point tests"""

        tf = TableFunction.TableFunction([1.0,2.0],[10.0,20.0],
                                         "LINEAR","NONE")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,None)

        x = 1.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 2.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,None)

        tf.Configure([1.0,2.0],[10.0,20.0],"LINEAR","CONSTANT")

        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 2.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        tf.Configure([1.0,2.0],[10.0,20.0],"LINEAR","LINEAR")

        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

        x = 1.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 2.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,25.0)

        tf.Configure([1.0,2.0],[10.0,20.0],"SPLINE","LINEAR")

        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

        x = 1.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 2.0 ;
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5 ;
        y = tf.GetValue(x)
        self.assertEqual(y,25.0)

class ThreeDataPoints(unittest.TestCase):
    def testThreeDataPoints(self):
        """three data point tests"""

        tf = TableFunction.TableFunction([1.0,3.0,2.0],[10.0,10.0,20.0],
                                         "LINEAR","LINEAR")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

        x = 1.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 2.0
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5
        y = tf.GetValue(x)
        self.assertEqual(y,15.0)

        x = 3.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 3.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

        tf.Configure([1.0,3.0,2.0],[10.0,10.0,20.0],"SPLINE","LINEAR")
        x = 0.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

        x = 1.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 1.5
        y = tf.GetValue(x)
        self.assertEqual(y,15.125)

        x = 2.0
        y = tf.GetValue(x)
        self.assertEqual(y,20.0)

        x = 2.5
        y = tf.GetValue(x)
        self.assertEqual(y,15.125)

        x = 3.0
        y = tf.GetValue(x)
        self.assertEqual(y,10.0)

        x = 3.5
        y = tf.GetValue(x)
        self.assertEqual(y,5.0)

class BugDataPoints(unittest.TestCase):
    def testBug1DataPoints(self):
        """bug 1 data point tests"""

        tf = TableFunction.TableFunction(
              [2.0, 2.1000000000000001, 22.0, 22.100000000000001],
              [6.0, 11.0, 11.0, 16.0], "LINEAR","CONSTANT")
        x = 2.0
        y = tf.GetValue(x)
        self.assertEqual(y,6.0)


if __name__ == "__main__":
    unittest.main()


