
#include "TMatrix.hh"
#include "TVector.hh"
#include "IntegrationPts.hh"
#include "ShapeFunc.hh"
#include "Element.hh"
#include "MaterialMat.hh"
#include "SparseCG.hh"

#include <fstream>

using FemLib::Element ;
using FemLib::ElementFactory ;
using FemLib::MaterialMat ;
using FemLib::TMatrix ;
using FemLib::TVector ;
using FemLib::Vec3D ;

void PrintDMtx(ostream &out,TMatrix<double> &mtx) ;
void PrintMtxSeq(ostream &out,TMatrix<double> &mtx) ;

static ofstream dsp("disp.dsp") ;

int main(int argc,char *argv[])
{
    bool two = true ;
    //bool two = false ;
    int start = 0 ;
    if (argc > 1) sscanf(argv[1],"%d",&start) ;

    TVector<Vec3D> coords(13) ;
    coords[ 0] = Vec3D(0.0,0.0,0.0) ;
    coords[ 1] = Vec3D(4.0,0.0,0.0) ;
    coords[ 2] = Vec3D(4.0,0.0,4.0) ;
    coords[ 3] = Vec3D(0.0,0.0,4.0) ;
    coords[ 4] = Vec3D(2.0,4.0,2.0) ;
    coords[ 5] = Vec3D(2.0,0.0,0.0) ;
    coords[ 6] = Vec3D(4.0,0.0,2.0) ;
    coords[ 7] = Vec3D(2.0,0.0,4.0) ;
    coords[ 8] = Vec3D(0.0,0.0,2.0) ;
    coords[ 9] = Vec3D(1.0,2.0,1.0) ;
    coords[10] = Vec3D(3.0,2.0,1.0) ;
    coords[11] = Vec3D(3.0,2.0,3.0) ;
    coords[12] = Vec3D(1.0,2.0,3.0) ;

    Element *elem = ElementFactory::Instance()->
                         CreateElement("Pyramid2IShapeFunc") ;

    double area = elem->Area(0,coords) ;
    cout << "Area 0: " << area << endl ;

    area = elem->Area(1,coords) ;
    cout << "Area 1: " << area << endl ;

    TVector<Vec3D> tcoords(13) ;
    coords[ 0] = Vec3D(0.0,0.0,0.0) ;
    coords[ 1] = Vec3D(4.0,0.0,0.0) ;
    coords[ 2] = Vec3D(4.0,0.0,4.0) ;
    coords[ 3] = Vec3D(0.0,0.0,4.0) ;
    coords[ 4] = Vec3D(2.0,4.0,2.0) ;
    coords[ 5] = Vec3D(2.0,0.0,0.0) ;
    coords[ 6] = Vec3D(4.0,0.0,2.0) ;
    coords[ 7] = Vec3D(2.0,0.0,4.0) ;
    coords[ 8] = Vec3D(0.0,0.0,2.0) ;
    coords[ 9] = Vec3D(1.0,2.0,1.0) ;
    coords[10] = Vec3D(3.0,2.0,1.0) ;
    coords[11] = Vec3D(3.0,2.0,3.0) ;
    coords[12] = Vec3D(1.0,2.0,3.0) ;

#if 0
    TVector<Vec3D> coords(20) ;
    coords[0] = Vec3D(-1.0,-1.0,-1.0) ;
    coords[1] = Vec3D(-1.0,-1.0, 1.0) ;
    coords[2] = Vec3D(-1.0, 1.0,-1.0) ;
    coords[3] = Vec3D(-1.0, 1.0, 1.0) ;
    coords[4] = Vec3D( 1.0,-1.0,-1.0) ;
    coords[5] = Vec3D( 1.0,-1.0, 1.0) ;
    coords[6] = Vec3D( 1.0, 1.0,-1.0) ;
    coords[7] = Vec3D( 1.0, 1.0, 1.0) ;

    coords[8] = Vec3D(-1.0,-1.0, 0.0) ;
    coords[9] = Vec3D(-1.0, 1.0, 0.0) ;
    coords[10] = Vec3D( 1.0,-1.0, 0.0) ;
    coords[11] = Vec3D( 1.0, 1.0, 0.0) ;

    coords[12] = Vec3D(-1.0, 0.0,-1.0) ;
    coords[13] = Vec3D(-1.0, 0.0, 1.0) ;
    coords[14] = Vec3D( 1.0, 0.0,-1.0) ;
    coords[15] = Vec3D( 1.0, 0.0, 1.0) ;

    coords[16] = Vec3D( 0.0,-1.0,-1.0) ;
    coords[17] = Vec3D( 0.0,-1.0, 1.0) ;
    coords[18] = Vec3D( 0.0, 1.0,-1.0) ;
    coords[19] = Vec3D( 0.0, 1.0, 1.0) ;

    TMatrix<double> mat = MaterialMat::Instance()->
                              IsotropicMatMatrix(10.0,0.3) ;

    Element *elem = ElementFactory::Instance()->
                         CreateElement("Brick2IShapeFunc") ;

    int size = two ? 32 : 20 ;
    SparseCGElasticity cg(size,start) ;
//    cg.SetTolerance(1e-15) ;
    int nodes[20],nodes2[20] ;
    for (int i=0 ; i<20 ; ++i) nodes[i] = i ;
    cg.ElementEquations(20,nodes) ;

    nodes2[0] = 4 ; nodes2[1] = 5 ; nodes2[2] = 6 ;
    nodes2[3] = 7 ; nodes2[4] = 20 ; nodes2[5] = 21 ;
    nodes2[6] = 22 ; nodes2[7] = 23 ; nodes2[8] = 10 ;
    nodes2[9] = 11 ; nodes2[10] = 24 ; nodes2[11] = 25 ;
    nodes2[12] = 14 ; nodes2[13] = 15 ; nodes2[14] = 26 ;
    nodes2[15] = 27 ; nodes2[16] = 28 ; nodes2[17] = 29 ;
    nodes2[18] = 30 ; nodes2[19] = 31 ;
    if (two) cg.ElementEquations(20,nodes2) ;


    TMatrix<double> k = elem->DeformationStiffMtx(coords,mat) ;
    cg.ElementValues(20,nodes,k) ;

    TVector<Vec3D> coords2(20) ;

    coords2[0] = Vec3D( 1.0,-1.0,-1.0) ;
    coords2[1] = Vec3D( 1.0,-1.0, 1.0) ;
    coords2[2] = Vec3D( 1.0, 1.0,-1.0) ;
    coords2[3] = Vec3D( 1.0, 1.0, 1.0) ;
    coords2[4] = Vec3D( 3.0,-1.0,-1.0) ;
    coords2[5] = Vec3D( 3.0,-1.0, 1.0) ;
    coords2[6] = Vec3D( 3.0, 1.0,-1.0) ;
    coords2[7] = Vec3D( 3.0, 1.0, 1.0) ;

    coords2[8] = Vec3D( 1.0,-1.0, 0.0) ;
    coords2[9] = Vec3D( 1.0, 1.0, 0.0) ;
    coords2[10] = Vec3D( 3.0,-1.0, 0.0) ;
    coords2[11] = Vec3D( 3.0, 1.0, 0.0) ;

    coords2[12] = Vec3D( 1.0, 0.0,-1.0) ;
    coords2[13] = Vec3D( 1.0, 0.0, 1.0) ;
    coords2[14] = Vec3D( 3.0, 0.0,-1.0) ;
    coords2[15] = Vec3D( 3.0, 0.0, 1.0) ;

    coords2[16] = Vec3D( 2.0,-1.0,-1.0) ;
    coords2[17] = Vec3D( 2.0,-1.0, 1.0) ;
    coords2[18] = Vec3D( 2.0, 1.0,-1.0) ;
    coords2[19] = Vec3D( 2.0, 1.0, 1.0) ;

    k = elem->DeformationStiffMtx(coords2,mat) ;
    if (two) cg.ElementValues(20,nodes2,k) ;

    Vec3D tract(1.0,0.0,0.0) ;
    if (two) {
        TVector<Vec3D> force = elem->IntegrateSurfaceVector(1,
                                    coords2,tract,9) ;
        cg.RhsValues(20,nodes2,force) ;
    } else {
        TVector<Vec3D> force = elem->IntegrateSurfaceVector(1,
                                    coords,tract,9) ;
        cg.RhsValues(20,nodes,force) ;
    }

    double values[3] = { 0.0, 0.0, 0.0 } ;
    cg.AddPrescribedValue(0,true,true,true,values) ;
    cg.AddPrescribedValue(1,true,true,false,values) ;
    cg.AddPrescribedValue(2,true,false,true,values) ;
    cg.AddPrescribedValue(3,true,false,false,values) ;
    cg.AddPrescribedValue(4,false,true,true,values) ;
    cg.AddPrescribedValue(5,false,true,false,values) ;
    cg.AddPrescribedValue(6,false,false,true,values) ;
    cg.AddPrescribedValue(8,true,true,false,values) ;
    cg.AddPrescribedValue(9,true,false,false,values) ;
    cg.AddPrescribedValue(10,false,true,false,values) ;
    cg.AddPrescribedValue(12,true,false,true,values) ;
    cg.AddPrescribedValue(13,true,false,false,values) ;
    cg.AddPrescribedValue(14,false,false,true,values) ;
    cg.AddPrescribedValue(16,false,true,true,values) ;
    cg.AddPrescribedValue(17,false,true,false,values) ;
    cg.AddPrescribedValue(18,false,false,true,values) ;
    if (two) {
        cg.AddPrescribedValue(20,false,true,true,values) ;
        cg.AddPrescribedValue(21,false,true,false,values) ;
        cg.AddPrescribedValue(22,false,false,true,values) ;
        cg.AddPrescribedValue(24,false,true,false,values) ;
        cg.AddPrescribedValue(26,false,false,true,values) ;
        cg.AddPrescribedValue(28,false,true,true,values) ;
        cg.AddPrescribedValue(29,false,true,false,values) ;
        cg.AddPrescribedValue(30,false,false,true,values) ;
    }

//     for (int i=0 ; i<size*3 ; ++i) {
//         for (int j=0 ; j<=i ; ++j) {
//             cout << i << ' ' << j << ' ' << cg.GetEntry(i,j) << endl ;
//         }
//     }

    // add fixed nodes

    double *x = cg.DoSolution() ;

    cout << "Displacements: " << endl ;
    for (int i=0 ; i<size ; ++i) {
        cout << i << ' ' << x[i*3] << ' ' << x[i*3+1] ;
        cout << ' ' << x[i*3+2] << endl ;
        dsp << i << ' ' << x[i*3] << ' ' << x[i*3+1] ;
        dsp << ' ' << x[i*3+2] << endl ;
    }
#endif

#if 0
    TMatrix<double> mat = MaterialMat::Instance()->
                              IsotropicMatMatrix(10.0,0.3) ;
    cout << "Isotropic:" << endl ;
    cout << mat << endl ;

    // test orthotropic

    double angles[3] = { 0.0, 0.0, 0.0 } ;
    mat = MaterialMat::Instance()->
              OrthotropicMatMatrixFromEngngConstants(
              10.0,8.0,6.0,0.1,0.2,0.3,4.0,5.0,6.0,angles) ;
    cout << "Ortho from conts:" << endl ;
    cout << mat << endl ;

    double cprops[9] ;
    cprops[0] = mat(0,0) ;
    cprops[1] = mat(1,1) ;
    cprops[2] = mat(2,2) ;
    cprops[3] = mat(0,1) ;
    cprops[4] = mat(1,2) ;
    cprops[5] = mat(0,2) ;
    cprops[6] = mat(3,3) ;
    cprops[7] = mat(4,4) ;
    cprops[8] = mat(5,5) ;
    mat = MaterialMat::Instance()->
              OrthotropicMatMatrixFromStiffnessComps(cprops,angles) ;
    cout << "Ortho from c's:" << endl ;
    cout << mat << endl ;

    double sprops[9] ;
    sprops[0] = 1.0/10.0 ;
    sprops[1] = 1.0/8.0 ;
    sprops[2] = 1.0/6.0 ;
    sprops[3] = -0.1/10.0 ;
    sprops[4] = -0.2/8.0 ;
    sprops[5] = -0.3/10.0 ;
    sprops[6] = 1.0/4.0 ;
    sprops[7] = 1.0/5.0 ;
    sprops[8] = 1.0/6.0 ;
    mat = MaterialMat::Instance()->
              OrthotropicMatMatrixFromComplianceComps(sprops,angles) ;
    cout << "Ortho from s's:" << endl ;
    cout << mat << endl ;

    angles[0] = 1.570796327 ;
    mat = MaterialMat::Instance()->
              OrthotropicMatMatrixFromComplianceComps(sprops,angles) ;
    cout << "rotated:" << endl ;
    cout << mat << endl ;
#endif


#if 0
//    IntegrationPts *pts = IntegrationPtsFactory::Instance()->
//        CreateIntegrationPts("TetIntegrationPts") ;
//    ShapeFunc *shape = ShapeFuncFactory::Instance()->
//        CreateShapeFunc("Tet2IShapeFunc") ;

//    Element *elem = ElementFactory::Instance()->
//                         CreateElement("Brick2IShapeFunc") ;
//    Element *elem = ElementFactory::Instance()->
//                         CreateElement("Tet2IShapeFunc") ;
    Element *elem = ElementFactory::Instance()->
                         CreateElement("Tet1IShapeFunc") ;

//        CreateElement("Tet2IShapeFunc","TetIntegrationPts") ;

    cout << endl << elem << endl << endl ;

//     TVector<Vec3D> coords(10) ;
//     coords[0] = Vec3D(0.0,0.0,0.0) ;
//     coords[1] = Vec3D(0.0,0.0,2.0) ;
//     coords[2] = Vec3D(0.0,2.0,2.0) ;
//     coords[3] = Vec3D(2.0,0.0,2.0) ;
//     coords[4] = Vec3D(0.0,0.0,1.0) ;
//     coords[5] = Vec3D(0.0,1.0,2.0) ;
//     coords[6] = Vec3D(0.0,1.0,1.0) ;
//     coords[7] = Vec3D(1.0,1.0,2.0) ;
//     coords[8] = Vec3D(1.0,0.0,2.0) ;
//     coords[9] = Vec3D(1.0,0.0,1.0) ;
//     coords[0] = Vec3D(0.0,0.0,0.0) ;

//     coords[1] = Vec3D(0.0,0.0,1.0) ;
//     coords[2] = Vec3D(0.0,1.0,1.0) ;
//     coords[3] = Vec3D(1.0,0.0,1.0) ;
//     coords[4] = Vec3D(0.0,0.0,0.5) ;
//     coords[5] = Vec3D(0.0,0.5,1.0) ;
//     coords[6] = Vec3D(0.0,0.5,0.5) ;
//     coords[7] = Vec3D(0.5,0.5,1.0) ;
//     coords[8] = Vec3D(0.5,0.0,1.0) ;
//     coords[9] = Vec3D(0.5,0.0,0.5) ;

//     TVector<Vec3D> coords(20) ;
//     coords[0] = Vec3D(-1.0,-1.0,-1.0) ;
//     coords[1] = Vec3D(-1.0,-1.0, 1.0) ;
//     coords[2] = Vec3D(-1.0, 1.0,-1.0) ;
//     coords[3] = Vec3D(-1.0, 1.0, 1.0) ;
//     coords[4] = Vec3D( 1.0,-1.0,-1.0) ;
//     coords[5] = Vec3D( 1.0,-1.0, 1.0) ;
//     coords[6] = Vec3D( 1.0, 1.0,-1.0) ;
//     coords[7] = Vec3D( 1.0, 1.0, 1.0) ;
// 
//     coords[8] = Vec3D(-1.0,-1.0, 0.0) ;
//     coords[9] = Vec3D(-1.0, 1.0, 0.0) ;
//     coords[10] = Vec3D( 1.0,-1.0, 0.0) ;
//     coords[11] = Vec3D( 1.0, 1.0, 0.0) ;
// 
//     coords[12] = Vec3D(-1.0, 0.0,-1.0) ;
//     coords[13] = Vec3D(-1.0, 0.0, 1.0) ;
//     coords[14] = Vec3D( 1.0, 0.0,-1.0) ;
//     coords[15] = Vec3D( 1.0, 0.0, 1.0) ;
// 
//     coords[16] = Vec3D( 0.0,-1.0,-1.0) ;
//     coords[17] = Vec3D( 0.0,-1.0, 1.0) ;
//     coords[18] = Vec3D( 0.0, 1.0,-1.0) ;
//     coords[19] = Vec3D( 0.0, 1.0, 1.0) ;

//     TVector<Vec3D> coords(10) ;
//     coords[0] = Vec3D( 8.0,2.0,2.0) ;
//     coords[1] = Vec3D(10.0,0.0,2.0) ;
//     coords[2] = Vec3D(10.0,2.0,2.0) ;
//     coords[3] = Vec3D(10.0,2.0,0.0) ;
//     coords[4] = Vec3D( 9.0,1.0,2.0) ;
//     coords[5] = Vec3D(10.0,1.0,2.0) ;
//     coords[6] = Vec3D( 9.0,2.0,2.0) ;
//     coords[7] = Vec3D(10.0,2.0,1.0) ;
//     coords[8] = Vec3D(10.0,1.0,1.0) ;
//     coords[9] = Vec3D( 9.0,2.0,1.0) ;

//     TVector<Vec3D> coords(10) ;
//     coords[0] = Vec3D(0.4162290096282959,1.8619699478149414,0.0) ;
//     coords[1] = Vec3D(0.34185799956321716,1.9705699682235718,0.085013300180435181) ;
//     coords[2] = Vec3D(0.39018100500106812,1.9615700244903564,0) ;
//     coords[3] = Vec3D(0.43815198540687561,1.9514199495315552,0.085013300180435181) ;
//     coords[4] = 0.5*(coords[0]+coords[1]) ;
//     coords[5] = 0.5*(coords[1]+coords[2]) ;
//     coords[6] = 0.5*(coords[2]+coords[0]) ;
//     coords[7] = 0.5*(coords[2]+coords[3]) ;
//     coords[8] = 0.5*(coords[1]+coords[3]) ;
//     coords[9] = 0.5*(coords[0]+coords[3]) ;

    TVector<Vec3D> coords(10) ;
    coords[0] = Vec3D(0.4162290096282959,1.8619699478149414,0.0) ;
    coords[1] = Vec3D(0.34185799956321716,1.9705699682235718,0.085013300180435181) ;
    coords[2] = Vec3D(0.39018100500106812,1.9615700244903564,0) ;
    coords[3] = Vec3D(0.43815198540687561,1.9514199495315552,0.085013300180435181) ;

    Vec3D cart(0.390180558, 1.96157014, 7.8082428e-7) ;
    Vec3D nat ;
    bool found = elem->FindNaturalCoords(cart,coords,nat) ;

    cout << "Natural: " << nat << endl ;


    double E = 1000.0 ;
    double nu = 0.3 ;

    double D11 = (E * (1.0 - nu))/ ((1.0 + nu) * (1.0 - 2.0*nu)) ;
    double D12 = D11 * (nu/(1.0-nu)) ;
    double D33 = E / (2 * (1 + nu)) ;

    TMatrix<double> mat(6,6) ;

    mat(0,0) = D11 ;
    mat(1,1) = D11 ;
    mat(2,2) = D11 ;

    mat(3,3) = D33 ;
    mat(4,4) = D33 ;
    mat(5,5) = D33 ;

    mat(0,1) = D12 ;
    mat(0,2) = D12 ;
    mat(1,0) = D12 ;
    mat(1,2) = D12 ;
    mat(2,0) = D12 ;
    mat(2,1) = D12 ;


    TMatrix<double> cond(3,3) ;
    cond(0,0) = 1.0 ;
    cond(1,1) = 1.0 ;
    cond(2,2) = 1.0 ;

//    TMatrix<double> k = elem->StiffMtxCheck(
//                                coords,&mat,true) ;
//    TMatrix<double> k = elem->DeformationStiffMtx(
//                                coords,&mat,true) ;
//    TMatrix<double> k = elem->ScalarFieldDerivMtx(
//                                coords,&cond,true) ;

//     double fact = 0.5 ;
//     TMatrix<double> k = elem->ScalarFieldShapeMtx(
//                                 coords,&fact,true) ;
// 
//     PrintMtxSeq(cout,k) ;
// 
//     cout << endl << endl ;

//    TMatrix<double> s = elem->DeformationStiffMtx(
//                                coords,&mat,true) ;

//    PrintDMtx(cout,s) ;

//     TVector<double> tvec(4) ;
//     for (int i=0 ; i<4 ; ++i) tvec(i) = i ;
// 
//     cout << tvec << endl ;
// 
//     cout << tvec * 4.0 << endl ;
// 
//     cout << 4.0 * tvec << endl ;
// 
//     cout << tvec * tvec << endl ;
// 
//     tvec *= 2.0 ;
// 
//     cout << tvec << endl ;
// 
//     // Matrix Tests
// 
//     TMatrix<double> tmat(4,3) ;
// 
//     TVector<double> tvec2(3) ;
//     for (int i=0 ; i<3 ; ++i) tvec2(i) = i ;
// 
//     int cur = 0 ;
//     for (int i=0 ; i<4 ; ++i) {
//         for (int j=0 ; j<3 ; ++j) {
//             tmat(i,j) = cur ;
//             ++cur ;
//         }
//     }
// 
//     cout << "mult: " << endl ;
//     cout << tmat << endl ;
//     cout << tvec2 << endl ;
//     cout << tmat * tvec2 << endl ;
// 

//     TMatrix<double> scale(3,3) ;
//     for (int i=0 ; i<3 ; ++i) scale(i,i) = 2.0 ;
// 
//     cout << tmat << endl ;
// 
//     tmat *= scale ;
// 
//     cout << tmat << endl ;
// 
//     TMatrix<double> mlt = tmat * scale ;
// 
//     cout << mlt << endl ;
// 
//     TMatrix<double> mlt1 = tmat * 3.0 ;
//     cout << mlt1 << endl ;
// 
//     TMatrix<double> mlt2 = 3.0 * tmat ;
//     cout << mlt2 << endl ;
#endif
}

#include <iomanip>

void PrintDMtx(ostream &out,TMatrix<double> &mtx)
{
    for (int i=0 ; i<mtx.Rows() ; ++i) {
        for (int j=0 ; j<7 ; ++j) {
            out << setprecision(4) << setw(10) << mtx(i,j) << ' ' ;
        }
        out << endl ;
    }
}

void PrintMtxSeq(ostream &out,TMatrix<double> &mtx)
{
    for (int i=0 ; i<mtx.Rows() ; ++i) {
        for (int j=0 ; j<mtx.Cols() ; ++j) {
            out << "    (" << i << ',' << j << ") = " << mtx(i,j) << endl ;
        }
    }
}

